﻿import mx.containers.Window;

class com.macromedia.timeentry.Reconcile extends mx.core.View {
	
	var btnNext:mx.controls.Button;
	var btnPrevious:mx.controls.Button;
	var btnFind:mx.controls.Button;
	var btnFinished:mx.controls.Button;
	var taMsg:mx.controls.TextArea;
	var dsReconcile:mx.data.components.DataSet;
	var dsData:mx.data.components.DataSet;
	var dsErrors:Array;
	var parentWindow:Window;
	var txtErrorNum:TextField;
	
	/**
	  initialize class. 
	*/
	function Reconcile() {
		addEventListener("load", onLoad);
		//create pointer to Window class that this view is contained within
		parentWindow = Window(_parent);
	}

	/**
	  initialize components that live within this form.  Setup listeners for events that we are interested in 
	  responding to within the context of the Reconcile class.
	*/
	function onLoad(): Void {
		btnNext.addEventListener("click", function(evt: Object) { evt.target._parent.btnNextClickHandler(evt); } );
		btnPrevious.addEventListener("click", function(evt: Object) { evt.target._parent.btnPreviousClickHandler(evt); } );
		btnFind.addEventListener("click", function(evt: Object) { evt.target._parent.btnFindClickHandler(evt); } );
		btnFinished.addEventListener("click", function(evt: Object) { evt.target._parent.btnFinishedClickHandler(evt); } );
		dsReconcile.addEventListener("iteratorScrolled", function(evt: Object) {evt.target._parent.dsReconcileIteratorScrolledHandler(evt); } );
		loadErrors();
	}

	/**
		assign DeltaPacket and errors.  The loadErrors function that is called within the constructor uses the properties.
		These properties needed to be passed in this way based on limitations of the Window class implementation. When the Window
		component fires the complete event to signal that this movieclip is loaded it is not guaranteed that all of the
		components within the movieclip are available (in particular the DataSet component).  The only place that this is 
		guaranteed is within the onLoad event. Therefore, when we get the complete event we assign the properties expecting that
		the loadErrors function can make use of them when it is called within the constructor.  This is not probably the best
		solution since we are not guaranteed that the complete event will fire before the loadErrors() function but it seems to
		work for the purposes of this sample.
	*/
	function setReconcileData(ds:mx.data.components.DataSet, deltaErrors:Array) {
		dsData = ds;
		dsErrors = deltaErrors;
	}

	/**
	  convert error array into a DataSet containing relevant information for reconciling
	*/
	function loadErrors():Void {
		for (var i:Number = 0; i<dsErrors.length; i++) {
			dsReconcile.addItem( {recId:dsErrors[i].getId(), errorMsg:dsErrors[i].getMessage(), operation:dsErrors[i].getOperation(), discardChange:false} );
		}
		dsReconcile.first();
		updateDisplay();
	}

	/**
	  find the record in the Time Entry Dataset to which the current error is associated.
	*/
	function btnFindClickHandler(evt:Object):Void {
		dsData.locateById(dsReconcile.recId);
	}

	/**
	  reconcile and close the parent Window.
	*/
	function btnFinishedClickHandler(evt:Object):Void {
		reconcileChanges();
		parentWindow.deletePopUp();
	}
		
	/**
	  move to next error.
	*/
	function btnNextClickHandler(evt:Object):Void {
		dsReconcile.next();
	}

	/**
	  move to previous error.
	*/
	function btnPreviousClickHandler(evt:Object):Void {
		dsReconcile.previous();
	}
	
	/**
	  update the display each time a new error is selected.
	*/
	function dsReconcileIteratorScrolledHandler(evt:Object):Void {
		updateDisplay();
	}
	
	/**
	  this function is called when the user closes the Reconcile dialog.  It loops through each record in the dsReconcile DataSet
	  and makes the appropriate call to undo the change. This functionality only takes place if the user chooses the 
	  "discard the change" CheckBox.
	*/
	function reconcileChanges():Void {
		var currDelta:mx.data.components.datasetclasses.Delta = null;
		dsReconcile.disableEvents();
		dsData.disableEvents();
		dsReconcile.first();
		while ( dsReconcile.hasNext() ) {
			if (dsReconcile.discardChange) {
				currDelta = dsErrors[dsReconcile.selectedIndex];
				switch( currDelta.getOperation() ) {
					case mx.data.components.datasetclasses.DeltaPacketConsts.Added:
						undoAddRecord(currDelta);
						break;
					case mx.data.components.datasetclasses.DeltaPacketConsts.Removed:
						undoRemoveRecord(currDelta);
						break;
					case mx.data.components.datasetclasses.DeltaPacketConsts.Modified:
						undoModifiedRecord(currDelta);
						break;
				} //switch
			} //if
			dsReconcile.next();
		} //while
		dsReconcile.enableEvents();
		dsData.enableEvents();
	}
	
	/**
	  removes an inserted record from the DataSet.  The act of removing the record from the DataSet will cancel out the insert
	  instruction that is in the DeltaPacket.  Therefore, it will not be resent to the server when the user presses save.
	*/
	function undoAddRecord(delta:mx.data.components.datasetclasses.Delta):Void {
		var recId = delta.getId();
		//remove inserted record from DataSet.  DeltaPacket should be automatically cleared
		if ( dsData.locateById( recId ) ) {
			dsData.removeItem();
		}
	}

	/**
	  Undo a delete operation by reinserting the deleted record into the DataSet.  We do not want this insert to be recorded in the DeltaPacket, 
	  therefore we must insert the record with logChanges turned off.
	*/
	function undoRemoveRecord(delta:mx.data.components.datasetclasses.Delta):Void {
		var recId = delta.getId();
		var src = delta.getSource();
		//add logic for inserting record
		dsData.logChanges = false;
		dsData.addItem();
		for (var fld in src) {
			dsData[fld] = src[fld];
		} //for
		dsData.logChanges = true;
		dsData.clearDelta( recId );
	}

	/**
	  change a modified record back to its original state.  The act of reverting the record in the DataSet will cancel out the modification
	  instruction that is in the DeltaPacket.  Therefore, it will not be resent to the server when the user presses save.
	*/
	function undoModifiedRecord(delta:mx.data.components.datasetclasses.Delta):Void {
		var recId = delta.getId();
		var src = delta.getSource();
		//Changed modified record back to original.  DeltaPacket should be automatically cleared
		if ( dsData.locateById( recId ) ) {
			for (var fld in src) {
				dsData[fld] = src[fld];
			}
		}
	}
	
	/**
	  disable/enable buttons and update error count based on current error within the dsReconcile DataSet.
	*/
	function updateDisplay() {
		btnNext.enabled = dsReconcile.selectedIndex < dsReconcile.length-1;
		btnPrevious.enabled = dsReconcile.selectedIndex > 0;
		txtErrorNum.text = "(" + (dsReconcile.selectedIndex+1) + " of " + dsReconcile.length + ")";
	}
}